@@ -50,7 +50,7 @@ array set CGNSnodes {
   FamilyPointers               {1 DataArray_t C1 32 0}
   FlowEquationSet              {1 FlowEquationSet_t MT {} 0}
   FlowSolution                 {0 FlowSolution_t MT {} 1}
-  FlowSolutionsPointers        {1 DataArray_t C1 32 0}
+  FlowSolutionPointers         {1 DataArray_t C1 32 0}
   GasModel                     {1 GasModel_t C1 {} 0}
   GeometryEntity               {0 GeometryEntity_t MT {} 1}
   GeometryFile                 {1 GeometryFile_t C1 {} 0}
@@ -85,6 +85,8 @@ array set CGNSnodes {
   OriginLocation               {1 DataArray_t R4 {3 2} 0}
   OversetHoles                 {0 OversetHoles_t MT {} 1}
   ParentData                   {1 DataArray_t I4 {} 0}
+  ParentElements               {1 DataArray_t I4 {} 0}
+  ParentElementsPosition       {1 DataArray_t I4 {} 0}
   Periodic                     {1 Periodic_t MT {} 0}
   PointList                    {1 IndexArray_t I4 {} 0}
   PointListDonor               {1 IndexArray_t I4 {} 0}
@@ -224,7 +226,7 @@ array set CGNSnodeChildren {
   ZoneGridConnectivity_t {Descriptor GridConnectivity \
       GridConnectivity1to1 OversetHoles UserDefinedData}
   ZoneIterativeData_t {ArbitraryGridMotionPointers DataArray DataClass \
-      Descriptor DimensionalUnits FlowSolutionsPointers \
+      Descriptor DimensionalUnits FlowSolutionPointers \
       GridCoordinatesPointers RigidGridMotionPointers UserDefinedData}
   ZoneSubRegion_t {BCRegionName DataArray DataClass DimensionalUnits \
       Descriptor FamilyName GridConnectivityRegionName GridLocation \
@@ -469,7 +471,7 @@ Constant or Frozen}}
       {32,MaxNumberOfZones,NumberOfSteps} {pointers to Family_t nodes}}
   FlowEquationSet_t {FlowEquationSet FlowEquationSet_t {0,1} MT {} {} {}}
   FlowSolution_t {{user defined} FlowSolution_t {0,N} MT {} {} {}}
-  FlowSolutionsPointers {FlowSolutionsPointers DataArray_t {0,1} C1 2 \
+  FlowSolutionPointers {FlowSolutionPointers DataArray_t {0,1} C1 2 \
       {32,NumberOfSteps} {pointers to FlowSolution_t nodes}}
   GasModel_t {GasModel GasModel_t {0,1} C1 1 {length of string} \
       {one of: Null, UserDefined, Ideal, VanderWaals, CaloricallyPerfect,\
@@ -550,15 +552,19 @@ Constant or Frozen}}
   ParentData {ParentData DataArray_t {0,1} I4 2 {ElementSize,4} \
       {Parent1[ElementSize], Parent2[ElementSize], SideOfParent1[ElementSize],\
       SideOfParent2[ElementSize]}}
+  ParentElements {ParentElements DataArray_t {0,1} I4 2 {ElementSize,2} \
+      {Parent1[ElementSize], Parent2[ElementSize]}}
+  ParentElementsPosition {ParentElementsPosition DataArray_t {0,1} I4 2 \
+      {ElementSize,2} {SideOfParent1[ElementSize], SideOfParent2[ElementSize]}}
   Periodic_t {Periodic Periodic_t {0,1} MT {} {} {}}
   PointList {PointList IndexArray_t {0,1} I4 2 {IndexDimension,ListLength} \
       {list of indices for vertices or cells}}
   PointListDonor {PointListDonor IndexArray_t {0,1} I4 2 \
       {IndexDimension of donor,ListLength} \
       {list of indices for vertices or cells on adjoining patch}}
-  PointRange {PointRange IndexRange_t 1 I4 2 {IndexDimension,2} \
+  PointRange {PointRange IndexRange_t {0,1} I4 2 {IndexDimension,2} \
       {begin[IndexDimension], end[IndexDimension]}}
-  PointRangeDonor {PointRangeDonor IndexRange_t 1 I4 2 \
+  PointRangeDonor {PointRangeDonor IndexRange_t {0,1} I4 2 \
       {IndexDimension of donor,2} \
       {begin[IndexDimension], end[IndexDimension] for adjoining patch}}
   ReferenceState_t {ReferenceState ReferenceState_t {0,1} MT {} {} {}}
@@ -715,7 +721,7 @@ array set CGNSnodeRef {
   FamilyPointers {{} {physical dataarray}}
   FlowEquationSet_t {{floweqn FlowEquationSet} {equation flowequationset}}
   FlowSolution_t {{gridflow FlowSolution} {solution flowsolution}}
-  FlowSolutionsPointers {}
+  FlowSolutionPointers {}
   GasModel_t {{floweqn GasModel} {equation auxiliary}}
   GeometryEntity_t {{} {families geometry}}
   GeometryFile_t {}
@@ -749,6 +755,8 @@ array set CGNSnodeRef {
   OriginLocation {}
   OversetHoles_t {{cnct OversetHoles} {connectivity overset}}
   ParentData {{} {grid elements}}
+  ParentElements {{} {grid elements}}
+  ParentElementsPosition {{} {grid elements}}
   Periodic_t {{cnct Periodic} {connectivity connproperty}}
   PointList {}
   PointListDonor {}
@@ -858,7 +866,7 @@ proc cgns_init {{version ""}} {
       ZoneGridConnectivity_t {Descriptor GridConnectivity \
         GridConnectivity1to1 OversetHoles}
       ZoneIterativeData_t {ArbitraryGridMotionPointers DataArray DataClass \
-        Descriptor DimensionalUnits FlowSolutionsPointers \
+        Descriptor DimensionalUnits FlowSolutionPointers \
         GridCoordinatesPointers RigidGridMotionPointers}
     }
   } else {
@@ -917,7 +925,7 @@ proc cgns_init {{version ""}} {
       ZoneGridConnectivity_t {Descriptor GridConnectivity \
         GridConnectivity1to1 OversetHoles UserDefinedData}
       ZoneIterativeData_t {ArbitraryGridMotionPointers DataArray DataClass \
-        Descriptor DimensionalUnits FlowSolutionsPointers \
+        Descriptor DimensionalUnits FlowSolutionPointers \
         GridCoordinatesPointers RigidGridMotionPointers UserDefinedData}
     }
   }
@@ -1060,12 +1068,14 @@ element types are: 0:Null, 1:UserDefined, 2:NODE, 3:BAR_2, 4:BAR_3,\
       DiscreteData_t {DataClass Descriptor DimensionalUnits \
         DiscreteDataArray GridLocation PointList PointRange Rind \
         UserDefinedData}
+      Elements_t {Descriptor ElementConnectivity ElementRange \
+        ParentElements ParentElementsPosition Rind UserDefinedData}
       FamilyBC_t {FamilyBCDataSet}
       FlowSolution_t {DataClass Descriptor DimensionalUnits \
         GridLocation PointList PointRange Rind SolutionData \
         UserDefinedData}
       ZoneIterativeData_t {ArbitraryGridMotionPointers DataArray DataClass \
-        Descriptor DimensionalUnits FlowSolutionsPointers \
+        Descriptor DimensionalUnits FlowSolutionPointers \
         GridCoordinatesPointers RigidGridMotionPointers \
         ZoneGridConnectivityPointers ZoneSubRegionPointers UserDefinedData}
       Zone_t {ArbitraryGridMotion DataClass Descriptor DiscreteData \
@@ -1584,48 +1594,9 @@ proc cgns:format {text width} {
 }
 
 proc cgns:help {type} {
-  global CGNSdata HelpData tcl_platform
-  set htmlfile $HelpData(cgns)
-  if {$htmlfile == ""} {
-    errormsg "CGNS documentation URL not setup"
-  }
+  global CGNSdata
   set html [lindex $CGNSdata($type) 0]
-  set tag [lindex $CGNSdata($type) 1]
   if {$html == ""} return
-  set ext [string tolower [file extension $htmlfile]]
-  if {$HelpData(winhtml) && $ext == ".chm"} {
-    if {$HelpData(chmfile) != $htmlfile} {
-      if {[catch {WinHtml file $htmlfile} msg]} {
-        errormsg $msg
-        return
-      }
-      set HelpData(chmfile) $htmlfile
-    }
-    if {[catch {eval WinHtml topic "$type/$html.html" $tag} msg]} {
-      errormsg $msg
-    }
-    return
-  }
-  if {$HelpData(browser) == ""} {
-    errormsg "browser not set up"
-  } else {
-    set doc $htmlfile
-    if {$ext == ".html" || $ext == ".htm"} {
-      set n [string last / $htmlfile]
-      if {$n > 0} {set doc [string range $htmlfile 0 $n]}
-    }
-    if {[string index $doc end] != "/"} {append doc /}
-    append doc "$type/$html.html"
-    if {$tag != ""} {append doc "\#$tag"}
-    set cmd "$HelpData(browser) $HelpData(htmlopts) "
-    if {$tcl_platform(platform) == "windows"} {
-      append cmd "\"$doc\""
-    } else {
-      append cmd $doc
-    }
-    if {[catch {eval exec $cmd &} msg]} {
-      errormsg $msg
-    }
-  }
+  help_show "$type/$html.html" [lindex $CGNSdata($type) 1]
 }
 
