@@ -142,13 +142,15 @@ array set ProgData {
   case 0
   fromtop 0
   start /
+  winwidth 640
+  winheight 500
   seppos 0.4
   sepwd 7
   reg,file ".cgnstools"
   reg,base "HKEY_CURRENT_USER/Software/CGNS"
   reg,key "CGNSview"
-  reg,vals {file follow verify backup autoload maxsize maxcnt \
-            toolbar find case fromtop showlines}
+  reg,vals {file follow verify backup autoload maxsize maxcnt toolbar \
+            find case fromtop showlines winwidth winheight seppos}
   cgnsversion ""
   cgnscheck ""
   cgnsplot ""
@@ -226,8 +228,10 @@ array set Export {
   exefile ""
   options ""
   basenum ""
+  zonenum ""
   solnum ""
   basenum,flag 1
+  zonenum,flag 1
   solnum,flag 1
   ascii 0
   solution 1
@@ -431,9 +435,11 @@ $m add command -label "Unit Conversions..." -command units_convert
 #--- help menu
 
 set m [menubar_get Help]
-$m add command -label "CGNSview..." -command {help_show cgnsview}
-$m add command -label "Utilities..." -command {help_show utilities}
-$m add command -label "CGNS..." -command {help_show cgns}
+$m add command -label "CGNSview..." \
+  -command {help_show cgnstools/cgnsview/index.html}
+$m add command -label "Utilities..." \
+  -command {help_show cgnstools/utilities/index.html}
+$m add command -label "CGNS..." -command {help_show}
 $m add separator
 $m add command -label "Configure..." -command help_setup
 $m add separator
@@ -771,34 +777,30 @@ R0lGODlhEAAQALMAAAAAAIAAAACAAICAAAAAgIAAgACAgICAgMDAwP8AAAD/AP//AAAA//8A\
 /wD//////yH5BAEAAAgALAAAAAAQABAAQwQiEMlJq50kX5kJ1hvShd+4mSJ4qmTrXl28ehw7\
 t+j75joVAQA7}
 
-button $f.help -image img_help -takefocus 0 -command {help_show cgnsview}
+button $f.help -image img_help -takefocus 0 \
+  -command {help_show cgnstools/cgnsview/index.html}
 pack $f.help -side left -padx 5
 set_balloon $f.help Help
 
 proc help_menu {} {
-  global HelpData
-  set n 0
-  foreach i {cgnsview utilities cgns} {
-    if {[help_valid $i]} {
-      menubar_state Help normal $n
-      if {$i == "cgnsview"} {
-        .toolbar.but.help configure -state normal
-      }
-    } else {
-      menubar_state Help disabled $n
-      if {$i == "cgnsview"} {
-        .toolbar.but.help configure -state disabled
-      }
-    }
-    incr n
+  if [help_valid] {
+    menubar_state Help normal 0
+    menubar_state Help normal 1
+    menubar_state Help normal 2
+    .toolbar.but.help configure -state normal
+  } else {
+    menubar_state Help disabled 0
+    menubar_state Help disabled 1
+    menubar_state Help disabled 2
+    .toolbar.but.help configure -state disabled
   }
 }
 
-help_init {cgnsview CGNSview} {utilities Utilities} {cgns CGNS}
+help_init
 
 #---------- main window
 
-frame .main -width 640 -height 500
+frame .main -width $ProgData(winwidth) -height $ProgData(winheight)
 pack .main -side top -fill both -expand 1 -padx 5 -pady 5
 
 #--- window seperator
@@ -1149,6 +1151,8 @@ proc do_quit {} {
   global ProgData
   if {![remove_backup]} return
   catch CGIOclose
+  set ProgData(winwidth) [winfo width .main]
+  set ProgData(winheight) [winfo height .main]
   foreach i $ProgData(reg,vals) {
     catch {tclreg_set $ProgData(reg,key) $i $ProgData($i)}
   }
@@ -1169,11 +1173,11 @@ vJ5dnaamVsXGt3fJyr64ucZYPrSmli6ku7vYa9rX3OAJADs=}
 proc do_about {} {
   global ProgData
   dialog .about -1 -1 "About CGNSview" \
-"CGNSview Version 3.0
+"CGNSview Version 3.1
 CGNS Library Version $ProgData(libvers)
 
 Bruce Wedan
-brucewedan@gmail.com" img_about 0 Close
+leavingdust@gmail.com" img_about 0 Close
 }
 
 proc do_backup {} {
@@ -2916,6 +2920,9 @@ proc run_command {title cmd {height 10} {width 60}} {
   text $w.f.t -width $width -height $height -yscroll "$w.f.s set"
   pack $w.f.t -side top -fill both -expand 1
 
+  $w.f.t tag configure warning -foreground blue
+  $w.f.t tag configure error -foreground red
+
   frame $w.b
   pack $w.b -side bottom
   button $w.b.close -text Close -command "destroy $w" -state disabled
@@ -2944,7 +2951,13 @@ proc run_event {w f} {
   if {![eof $f]} {
     fconfigure $f -blocking 0
     while {[gets $f line] >= 0} {
-      $w.f.t insert end "$line\n"
+      if {[string match {WARNING:*} $line]} {
+        $w.f.t insert end "$line\n" warning
+      } elseif {[string match {ERROR:*} $line]} {
+        $w.f.t insert end "$line\n" error
+      } else {
+        $w.f.t insert end "$line\n"
+      }
     }
     fconfigure $f -blocking 1
     $w.f.t yview -pickplace end
