@@ -80,13 +80,17 @@ int main (int argc, char *argv[])
         int type = 0;
         int n = 0;
         if (argv[1][n] == '-') n++;
-        if (argv[1][n] == 'a' || argv[1][n] == 'A') {
-            type = CG_FILE_ADF;
-            strcpy (outfile, "test.cga");
+        if (argv[1][n] == 'a' || argv[1][n] == 'A' || argv[1][n] == '1') {
+            if (NULL != strchr(argv[1], '2'))
+                type = CG_FILE_ADF2;
+            else
+                type = CG_FILE_ADF;
         }
-        else if (argv[1][n] == 'h' || argv[1][n] == 'H') {
+        else if (argv[1][n] == 'h' || argv[1][n] == 'H' || argv[1][n] == '2') {
             type = CG_FILE_HDF5;
-            strcpy (outfile, "test.cgh");
+        }
+        else if (argv[1][n] == '3') {
+            type = CG_FILE_ADF2;
         }
         else {
             fprintf(stderr, "unknown option\n");
@@ -342,6 +346,7 @@ void write_equationset ()
     float mu = (float)1.716e-5;
     float exp = (float)0.666;
     float pt = (float)0.9;
+    float exps[5];
 
     for (n = 0; n < 6; n++)
         diff[n] = 0;
@@ -377,6 +382,11 @@ void write_equationset ()
 
     /* gas model */
 
+    exps[0] = (float)0.0;
+    exps[1] = (float)2.0;
+    exps[2] = (float)-2.0;
+    exps[3] = (float)-1.0;
+    exps[4] = (float)0.0;
     if (cg_goto(cgfile, cgbase, "FlowEquationSet_t", 1,
             "GasModel_t", 1, "end") ||
         cg_dataclass_write(CGNS_ENUMV(DimensionlessConstant)) ||
@@ -388,17 +398,32 @@ void write_equationset ()
             "GasModel_t", 1, "DataArray_t", 2, "end") ||
         cg_dataclass_write(CGNS_ENUMV(Dimensional)) ||
         cg_units_write(CGNS_ENUMV(Slug), CGNS_ENUMV(Foot),
-            CGNS_ENUMV(Second), CGNS_ENUMV(Rankine), CGNS_ENUMV(Radian)))
+            CGNS_ENUMV(Second), CGNS_ENUMV(Rankine), CGNS_ENUMV(Radian)) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("gas model");
 
     /* viscosity model */
 
+    exps[1] = (float)0.0;
+    exps[2] = (float)0.0;
+    exps[3] = (float)1.0;
     if (cg_goto(cgfile, cgbase, "FlowEquationSet_t", 1,
             "ViscosityModel_t", 1, "end") ||
         cg_array_write("SutherlandLawConstant", CGNS_ENUMV(RealSingle),
             1, &dim, &ts) ||
+        cg_gopath(cgfile, "SutherlandLawConstant") ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
+        error_exit("viscosity model");
+
+    exps[0] = (float)1.0;
+    exps[1] = (float)-1.0;
+    exps[2] = (float)-1.0;
+    exps[3] = (float)0.0;
+    if (cg_gopath(cgfile, "..") ||
         cg_array_write("ViscosityMolecularReference", CGNS_ENUMV(RealSingle),
-            1, &dim, &mu))
+            1, &dim, &mu) ||
+        cg_gopath(cgfile, "ViscosityMolecularReference") ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("viscosity model");
 
     /* thermal conductivity model */
@@ -427,6 +452,7 @@ void write_equationset ()
 void write_coords(int nz)
 {
     int k, nn, n, nij, koff, cgcoord;
+    float exps[5];
 
     koff = nz == 1 ? 1 - NUM_SIDE : 0;
     nij = NUM_SIDE * NUM_SIDE;
@@ -434,13 +460,25 @@ void write_coords(int nz)
         for (nn = 0; nn < nij; nn++)
             zcoord[n++] = (float)(k + koff);
     }
+    for (n = 0; n < 5; n++)
+        exps[n] = (float)0.0;
+    exps[1] = (float)1.0;
 
     if (cg_coord_write(cgfile, cgbase, nz, CGNS_ENUMV(RealSingle),
             "CoordinateX", xcoord, &cgcoord) ||
+        cg_goto(cgfile, cgbase, "Zone_t", nz, "GridCoordinates_t", 1,
+            "CoordinateX", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps) ||
         cg_coord_write(cgfile, cgbase, nz, CGNS_ENUMV(RealSingle),
             "CoordinateY", ycoord, &cgcoord) ||
+        cg_goto(cgfile, cgbase, "Zone_t", nz, "GridCoordinates_t", 1,
+            "CoordinateY", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps) ||
         cg_coord_write(cgfile, cgbase, nz, CGNS_ENUMV(RealSingle),
-            "CoordinateZ", zcoord, &cgcoord)) {
+            "CoordinateZ", zcoord, &cgcoord) ||
+        cg_goto(cgfile, cgbase, "Zone_t", nz, "GridCoordinates_t", 1,
+            "CoordinateZ", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps)) {
         sprintf (errmsg, "zone %d coordinates", nz);
         error_exit(errmsg);
     }
@@ -483,6 +521,7 @@ void write_structured()
     int transform[3];
     int cgsol, rind[6], cgfld;
     char name[33];
+    float exps[5];
 
     printf ("writing structured base\n");
     fflush (stdout);
@@ -579,7 +618,10 @@ void write_structured()
 
     if (cg_family_write(cgfile, cgbase, "WallFamily", &cgfam) ||
         cg_fambc_write(cgfile, cgbase, cgfam, "WallBC",
-            CGNS_ENUMV(BCWall), &cgbc))
+            CGNS_ENUMV(BCWall), &cgbc) ||
+        cg_goto(cgfile, cgbase, "Family_t", 1, "FamilyBC_t", 1, "end") ||
+        cg_bcdataset_write("WallBCDataSet", CGNS_ENUMV(BCWall),
+            CGNS_ENUMV(Neumann)))
         error_exit("wall family bc");
 
     /* write out some bogus geometry info for the family */
@@ -655,6 +697,10 @@ void write_structured()
 
     /* write solution for zone 1 as vertex with rind points */
 
+    exps[0] = (float)1.0;
+    exps[1] = (float)-3.0;
+    for (n = 2; n < 5; n++)
+        exps[n] = (float)0.0;
     for (n = 0; n < 6; n++)
         rind[n] = 1;
     if (cg_sol_write(cgfile, cgbase, 1, "VertexSolution",
@@ -662,7 +708,9 @@ void write_structured()
         cg_goto(cgfile, cgbase, "Zone_t", 1, "FlowSolution_t", cgsol, "end") ||
         cg_rind_write(rind) ||
         cg_field_write(cgfile, cgbase, 1, cgsol, CGNS_ENUMV(RealSingle),
-            "Density", solution, &cgfld))
+            "Density", solution, &cgfld) ||
+        cg_gorel(cgfile, "Density", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("zone 1 solution");
 
     /* write solution for zone 2 as cell center with rind points */
@@ -673,7 +721,9 @@ void write_structured()
         cg_goto(cgfile, cgbase, "Zone_t", 2, "FlowSolution_t", cgsol, "end") ||
         cg_rind_write(rind) ||
         cg_field_write(cgfile, cgbase, 2, cgsol, CGNS_ENUMV(RealSingle),
-            "Density", solution, &cgfld))
+            "Density", solution, &cgfld) ||
+        cg_gorel(cgfile, "Density", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("zone 2 solution");
 }
 
@@ -687,6 +737,7 @@ void write_unstructured()
     cgsize_t range[2];
     int cgsol, cgfld;
     char name[33];
+    float exps[5];
 #ifdef UNSTRUCTURED_1TO1
     int d_range[2], transform;
 #else
@@ -836,15 +887,25 @@ void write_unstructured()
 
     /* write solution for zone 1 as vertex and zone 2 as cell center */
 
+    exps[0] = (float)1.0;
+    exps[1] = (float)-3.0;
+    for (n = 2; n < 5; n++)
+        exps[n] = (float)0.0;
     if (cg_sol_write(cgfile, cgbase, 1, "VertexSolution",
             CGNS_ENUMV(Vertex), &cgsol) ||
         cg_field_write(cgfile, cgbase, 1, cgsol, CGNS_ENUMV(RealSingle),
-            "Density", solution, &cgfld))
+            "Density", solution, &cgfld) ||
+        cg_goto(cgfile, cgbase, "Zone_t", 1, "FlowSolution_t", cgsol,
+            "Density", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("zone 1 solution");
     if (cg_sol_write(cgfile, cgbase, 2, "CellCenterSolution",
             CGNS_ENUMV(CellCenter), &cgsol) ||
         cg_field_write(cgfile, cgbase, 2, cgsol, CGNS_ENUMV(RealSingle),
-            "Density", solution, &cgfld))
+            "Density", solution, &cgfld) ||
+        cg_goto(cgfile, cgbase, "Zone_t", 2, "FlowSolution_t", cgsol,
+            "Density", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("zone 2 solution");
 }
 
@@ -1082,6 +1143,7 @@ void write_mismatched()
     cgsize_t range[6], d_range[6];
     int transform[3];
     CGNS_ENUMT(PointSetType_t) d_type;
+    float exps[5];
 
     printf ("writing mismatched base\n");
     fflush (stdout);
@@ -1132,14 +1194,29 @@ void write_mismatched()
     }
     size[1] = nj;
     size[4] = nj - 1;
+    for (n = 0; n < 5; n++)
+        exps[n] = (float)0.0;
+    exps[1] = (float)1.0;
     if (cg_zone_write(cgfile, cgbase, "CylindricalZone", size,
             CGNS_ENUMV(Structured), &cgzone) ||
         cg_coord_write(cgfile, cgbase, cgzone, CGNS_ENUMV(RealSingle),
             "CoordinateR", xcoord, &cgcoord) ||
+        cg_goto(cgfile, cgbase, "Zone_t", cgzone, "GridCoordinates", 0,
+            "CoordinateR", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps) ||
         cg_coord_write(cgfile, cgbase, cgzone, CGNS_ENUMV(RealSingle),
+            "CoordinateZ", zcoord, &cgcoord) ||
+        cg_goto(cgfile, cgbase, "Zone_t", cgzone, "GridCoordinates", 0,
+            "CoordinateZ", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
+        error_exit("cylindrical zone");
+    exps[1] = (float)0.0;
+    exps[4] = (float)1.0;
+    if (cg_coord_write(cgfile, cgbase, cgzone, CGNS_ENUMV(RealSingle),
             "CoordinateTheta", ycoord, &cgcoord) ||
-        cg_coord_write(cgfile, cgbase, cgzone, CGNS_ENUMV(RealSingle),
-            "CoordinateZ", zcoord, &cgcoord))
+        cg_goto(cgfile, cgbase, "Zone_t", cgzone, "GridCoordinates", 0,
+            "CoordinateTheta", 0, NULL) ||
+        cg_exponents_write(CGNS_ENUMV(RealSingle), exps))
         error_exit("cylindrical zone");
 
     /* zone 1 -> zone 2 connectivity */
